@@ -10,14 +10,6 @@ const helpers = require("./helpers");
 // native application id
 var appID = "com.github.browserpass.native";
 
-// OTP extension id
-var otpID = [
-    "afjjoildnccgmjbblnklbohcbjehjaph", // webstore releases
-    "jbnpmhhgnchcoljeobafpinmchnpdpin", // github releases
-    "fcmmcnalhjjejhpnlfnddimcdlmpkbdf", // local unpacked
-    "browserpass-otp@maximbaz.com" // firefox
-];
-
 // default settings
 var defaultSettings = {
     autoSubmit: false,
@@ -25,7 +17,8 @@ var defaultSettings = {
     stores: {},
     foreignFills: {},
     username: null,
-    theme: "dark"
+    theme: "dark",
+    enableOTP: false,
 };
 
 var authListeners = {};
@@ -34,14 +27,14 @@ var badgeCache = {
     files: null,
     settings: null,
     expires: Date.now(),
-    isRefreshing: false
+    isRefreshing: false,
 };
 
 // the last text copied to the clipboard is stored here in order to be cleared after 60 seconds
 let lastCopiedText = null;
 
 chrome.browserAction.setBadgeBackgroundColor({
-    color: "#666"
+    color: "#666",
 });
 
 // watch for tab updates
@@ -59,7 +52,7 @@ chrome.tabs.onUpdated.addListener((tabId, info) => {
 });
 
 // handle incoming messages
-chrome.runtime.onMessage.addListener(function(message, sender, sendResponse) {
+chrome.runtime.onMessage.addListener(function (message, sender, sendResponse) {
     receiveMessage(message, sender, sendResponse);
 
     // allow async responses after this function returns
@@ -67,7 +60,7 @@ chrome.runtime.onMessage.addListener(function(message, sender, sendResponse) {
 });
 
 // handle keyboard shortcuts
-chrome.commands.onCommand.addListener(async command => {
+chrome.commands.onCommand.addListener(async (command) => {
     switch (command) {
         case "fillBest":
             try {
@@ -76,7 +69,7 @@ chrome.commands.onCommand.addListener(async command => {
                     // only fill on real domains
                     return;
                 }
-                handleMessage(settings, { action: "listFiles" }, listResults => {
+                handleMessage(settings, { action: "listFiles" }, (listResults) => {
                     const logins = helpers.prepareLogins(listResults.files, settings);
                     const bestLogin = helpers.filterSortLogins(logins, "", true)[0];
                     if (bestLogin) {
@@ -91,7 +84,7 @@ chrome.commands.onCommand.addListener(async command => {
 });
 
 // handle fired alarms
-chrome.alarms.onAlarm.addListener(alarm => {
+chrome.alarms.onAlarm.addListener((alarm) => {
     if (alarm.name === "clearClipboard") {
         if (readFromClipboard() === lastCopiedText) {
             copyToClipboard("", false);
@@ -133,13 +126,13 @@ async function updateMatchingPasswordsCount(tabId, forceRefresh = false) {
                 files: response.data.files,
                 settings: settings,
                 expires: Date.now() + CACHE_TTL_MS,
-                isRefreshing: false
+                isRefreshing: false,
             };
         }
 
         try {
             const tab = await chrome.tabs.get(tabId);
-            badgeCache.settings.origin = new URL(tab.url).origin;
+            badgeCache.settings.origin = new BrowserpassURL(tab.url).origin;
         } catch (e) {
             throw new Error(`Unable to determine domain of the tab with id ${tabId}`);
         }
@@ -155,7 +148,7 @@ async function updateMatchingPasswordsCount(tabId, forceRefresh = false) {
         // Set badge for the current tab
         chrome.browserAction.setBadgeText({
             text: "" + (matchedPasswordsCount || ""),
-            tabId: tabId
+            tabId: tabId,
         });
     } catch (e) {
         console.log(e);
@@ -174,7 +167,7 @@ async function updateMatchingPasswordsCount(tabId, forceRefresh = false) {
 function copyToClipboard(text, clear = true) {
     document.addEventListener(
         "copy",
-        function(e) {
+        function (e) {
             e.clipboardData.setData("text/plain", text);
             e.preventDefault();
         },
@@ -247,7 +240,7 @@ async function saveRecent(settings, login, remove = false) {
         const db = await idb.openDB("browserpass", DB_VERSION, {
             upgrade(db) {
                 db.createObjectStore("log", { keyPath: "time" });
-            }
+            },
         });
         await db.add("log", { time: Date.now(), host: settings.origin, login: login.login });
     } catch {
@@ -269,12 +262,12 @@ async function dispatchFill(settings, request, allFrames, allowForeign, allowNoS
     request = Object.assign(deepCopy(request), {
         allowForeign: allowForeign,
         allowNoSecret: allowNoSecret,
-        foreignFills: settings.foreignFills[settings.origin] || {}
+        foreignFills: settings.foreignFills[settings.origin] || {},
     });
 
     let perFrameResults = await chrome.tabs.executeScript(settings.tab.id, {
         allFrames: allFrames,
-        code: `window.browserpass.fillLogin(${JSON.stringify(request)});`
+        code: `window.browserpass.fillLogin(${JSON.stringify(request)});`,
     });
 
     // merge filled fields into a single array
@@ -313,36 +306,13 @@ async function dispatchFill(settings, request, allFrames, allowForeign, allowNoS
 async function dispatchFocusOrSubmit(settings, request, allFrames, allowForeign) {
     request = Object.assign(deepCopy(request), {
         allowForeign: allowForeign,
-        foreignFills: settings.foreignFills[settings.origin] || {}
+        foreignFills: settings.foreignFills[settings.origin] || {},
     });
 
-    let perFrameResults = await chrome.tabs.executeScript(settings.tab.id, {
+    await chrome.tabs.executeScript(settings.tab.id, {
         allFrames: allFrames,
-        code: `window.browserpass.focusOrSubmit(${JSON.stringify(request)});`
+        code: `window.browserpass.focusOrSubmit(${JSON.stringify(request)});`,
     });
-
-    // if necessary, dispatch Enter keypress to autosubmit the form
-    // currently only works on Chromium and requires debugger permission
-    try {
-        for (let frame of perFrameResults) {
-            if (frame.needPressEnter) {
-                chrome.debugger.attach({ tabId: settings.tab.id }, "1.2");
-                for (let type of ["keyDown", "keyUp"]) {
-                    chrome.debugger.sendCommand(
-                        { tabId: settings.tab.id },
-                        "Input.dispatchKeyEvent",
-                        {
-                            type: type,
-                            windowsVirtualKeyCode: 13,
-                            nativeVirtualKeyCode: 13
-                        }
-                    );
-                }
-                chrome.debugger.detach({ tabId: settings.tab.id });
-                break;
-            }
-        }
-    } catch (e) {}
 }
 
 /**
@@ -359,7 +329,7 @@ async function injectScript(settings, allFrames) {
         const waitTimeout = setTimeout(reject, MAX_WAIT);
         await chrome.tabs.executeScript(settings.tab.id, {
             allFrames: allFrames,
-            file: "js/inject.dist.js"
+            file: "js/inject.dist.js",
         });
         clearTimeout(waitTimeout);
         resolve(true);
@@ -392,9 +362,9 @@ async function fillFields(settings, login, fields) {
 
     // build fill request
     var fillRequest = {
-        origin: new URL(settings.tab.url).origin,
+        origin: new BrowserpassURL(settings.tab.url).origin,
         login: login,
-        fields: fields
+        fields: fields,
     };
 
     let allFrames = false;
@@ -479,9 +449,9 @@ async function fillFields(settings, login, fields) {
 
     // build focus or submit request
     let focusOrSubmitRequest = {
-        origin: new URL(settings.tab.url).origin,
+        origin: new BrowserpassURL(settings.tab.url).origin,
         autoSubmit: getSetting("autoSubmit", login, settings),
-        filledFields: filledFields
+        filledFields: filledFields,
     };
 
     // try to focus or submit form with the settings that were used to fill it
@@ -519,7 +489,7 @@ function getLocalSettings() {
 async function getFullSettings() {
     var settings = getLocalSettings();
     var configureSettings = Object.assign(deepCopy(settings), {
-        defaultStore: {}
+        defaultStore: {},
     });
     var response = await hostAction(configureSettings, "configure");
     if (response.status != "ok") {
@@ -550,7 +520,7 @@ async function getFullSettings() {
             settings.stores.default = {
                 id: "default",
                 name: "pass",
-                path: response.data.defaultStore.path
+                path: response.data.defaultStore.path,
             };
             var fileSettings = JSON.parse(response.data.defaultStore.settings);
             if (typeof settings.stores.default.settings !== "object") {
@@ -585,7 +555,6 @@ async function getFullSettings() {
     try {
         settings.tab = (await chrome.tabs.query({ active: true, currentWindow: true }))[0];
         let originInfo = new BrowserpassURL(settings.tab.url);
-        settings.host = originInfo.host; // TODO remove this after OTP extension is migrated
         settings.origin = originInfo.origin;
     } catch (e) {}
 
@@ -680,8 +649,8 @@ function handleModalAuth(requestDetails) {
     return {
         authCredentials: {
             username: this.login.fields.login,
-            password: this.login.fields.secret
-        }
+            password: this.login.fields.secret,
+        },
     };
 }
 
@@ -710,7 +679,7 @@ async function handleMessage(settings, message, sendResponse) {
     } catch (e) {
         sendResponse({
             status: "error",
-            message: "Unable to fetch and parse login fields: " + e.toString()
+            message: "Unable to fetch and parse login fields: " + e.toString(),
         });
         return;
     }
@@ -720,7 +689,7 @@ async function handleMessage(settings, message, sendResponse) {
         case "getSettings":
             sendResponse({
                 status: "ok",
-                settings: settings
+                settings: settings,
             });
             break;
         case "saveSettings":
@@ -730,7 +699,7 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: e.message
+                    message: e.message,
                 });
             }
             break;
@@ -745,7 +714,7 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: "Unable to enumerate password files" + e.toString()
+                    message: "Unable to enumerate password files" + e.toString(),
                 });
             }
             break;
@@ -757,7 +726,7 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: "Unable to copy password"
+                    message: "Unable to copy password",
                 });
             }
             break;
@@ -769,10 +738,32 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: "Unable to copy username"
+                    message: "Unable to copy username",
                 });
             }
             break;
+        case "copyOTP":
+            if (settings.enableOTP) {
+                try {
+                    if (!message.login.fields.otp) {
+                        throw new Exception("No OTP seed available");
+                    }
+                    copyToClipboard(helpers.makeTOTP(message.login.fields.otp.params));
+                    sendResponse({ status: "ok" });
+                } catch (e) {
+                    sendResponse({
+                        status: "error",
+                        message: "Unable to copy OTP token",
+                    });
+                }
+            } else {
+                sendResponse({ status: "error", message: "OTP support is disabled" });
+            }
+            break;
+
+        case "getDetails":
+            sendResponse({ status: "ok", login: message.login });
+            break;
 
         case "launch":
         case "launchInNewTab":
@@ -796,7 +787,7 @@ async function handleMessage(settings, message, sendResponse) {
                 }
                 authListeners[tab.id] = handleModalAuth.bind({
                     url: url,
-                    login: message.login
+                    login: message.login,
                 });
                 chrome.webRequest.onAuthRequired.addListener(
                     authListeners[tab.id],
@@ -807,7 +798,7 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: "Unable to launch URL: " + e.toString()
+                    message: "Unable to launch URL: " + e.toString(),
                 });
             }
             break;
@@ -821,11 +812,20 @@ async function handleMessage(settings, message, sendResponse) {
 
                 // no need to check filledFields, because fillFields() already throws an error if empty
                 sendResponse({ status: "ok", filledFields: filledFields });
+
+                // copy OTP token after fill
+                if (
+                    settings.enableOTP &&
+                    typeof message.login !== "undefined" &&
+                    message.login.fields.hasOwnProperty("otp")
+                ) {
+                    copyToClipboard(helpers.makeTOTP(message.login.fields.otp.params));
+                }
             } catch (e) {
                 try {
                     sendResponse({
                         status: "error",
-                        message: e.toString()
+                        message: e.toString(),
                     });
                 } catch (e) {
                     // TODO An error here is typically a closed message port, due to a popup taking focus
@@ -842,22 +842,17 @@ async function handleMessage(settings, message, sendResponse) {
             } catch (e) {
                 sendResponse({
                     status: "error",
-                    message: e.message
+                    message: e.message,
                 });
             }
             break;
         default:
             sendResponse({
                 status: "error",
-                message: "Unknown action: " + message.action
+                message: "Unknown action: " + message.action,
             });
             break;
     }
-
-    // trigger browserpass-otp
-    if (typeof message.login !== "undefined" && message.login.fields.hasOwnProperty("otp")) {
-        triggerOTPExtension(settings, message.action, message.login.fields.otp);
-    }
 }
 
 /**
@@ -873,7 +868,7 @@ async function handleMessage(settings, message, sendResponse) {
 function hostAction(settings, action, params = {}) {
     var request = {
         settings: settings,
-        action: action
+        action: action,
     };
     for (var key in params) {
         request[key] = params[key];
@@ -894,12 +889,14 @@ function hostAction(settings, action, params = {}) {
 async function parseFields(settings, login) {
     var response = await hostAction(settings, "fetch", {
         storeId: login.store.id,
-        file: login.login + ".gpg"
+        file: login.login + ".gpg",
     });
     if (response.status != "ok") {
         throw new Error(JSON.stringify(response)); // TODO handle host error
     }
 
+    var allowEmpty = ["login"];
+
     // save raw data inside login
     login.raw = response.data.contents;
 
@@ -908,30 +905,29 @@ async function parseFields(settings, login) {
         secret: ["secret", "password", "pass"],
         login: ["login", "username", "user"],
         openid: ["openid"],
-        otp: ["otp", "totp", "hotp"],
-        url: ["url", "uri", "website", "site", "link", "launch"]
+        otp: ["otp", "totp"],
+        url: ["url", "uri", "website", "site", "link", "launch"],
     };
     login.settings = {
-        autoSubmit: { name: "autosubmit", type: "bool" }
+        autoSubmit: { name: "autosubmit", type: "bool" },
     };
-    var lines = login.raw.split(/[\r\n]+/).filter(line => line.trim().length > 0);
-    lines.forEach(function(line) {
-        // check for uri-encoded otp
-        if (line.match(/^otpauth:\/\/.+/)) {
-            login.fields.otp = { key: null, data: line };
-            return;
+    var lines = login.raw.split(/[\r\n]+/).filter((line) => line.trim().length > 0);
+    lines.forEach(function (line) {
+        // check for uri-encoded otp without line prefix
+        if (line.match(/^otpauth:\/\/.+/i)) {
+            line = `otp: ${line}`;
         }
 
         // split key / value & ignore non-k/v lines
-        var parts = line.match(/^(.+?):(.+)$/);
+        var parts = line.match(/^(.+?):(.*)$/);
         if (parts === null) {
             return;
         }
         parts = parts
             .slice(1)
-            .map(value => value.trim())
-            .filter(value => value.length);
-        if (parts.length != 2) {
+            .map((value) => value.trim())
+            .filter((value) => value.length);
+        if (!parts.length) {
             return;
         }
 
@@ -941,11 +937,10 @@ async function parseFields(settings, login) {
                 Array.isArray(login.fields[key]) &&
                 login.fields[key].includes(parts[0].toLowerCase())
             ) {
-                if (key === "otp") {
-                    login.fields[key] = { key: parts[0].toLowerCase(), data: parts[1] };
-                } else {
-                    login.fields[key] = parts[1];
+                if (parts.length < 2 && !allowEmpty.includes(key)) {
+                    return;
                 }
+                login.fields[key] = parts[1];
                 break;
             }
         }
@@ -985,6 +980,41 @@ async function parseFields(settings, login) {
             delete login.settings[key];
         }
     }
+
+    // preprocess otp
+    if (settings.enableOTP && login.fields.hasOwnProperty("otp")) {
+        if (login.fields.otp.match(/^otpauth:\/\/.+/i)) {
+            // attempt to parse otp data as URI
+            try {
+                let url = new URL(login.fields.otp.toLowerCase());
+                let otpParts = url.pathname.split("/").filter((s) => s.trim());
+                login.fields.otp = {
+                    raw: login.fields.otp,
+                    params: {
+                        type: otpParts[0] === "otp" ? "totp" : otpParts[0],
+                        secret: url.searchParams.get("secret").toUpperCase(),
+                        algorithm: url.searchParams.get("algorithm") || "sha1",
+                        digits: parseInt(url.searchParams.get("digits") || "6"),
+                        period: parseInt(url.searchParams.get("period") || "30"),
+                    },
+                };
+            } catch (e) {
+                throw new Exception(`Unable to parse URI: ${otp.data}`, e);
+            }
+        } else {
+            // use default params for secret-only otp data
+            login.fields.otp = {
+                raw: login.fields.otp,
+                params: {
+                    type: "totp",
+                    secret: login.fields.otp.toUpperCase(),
+                    algorithm: "sha1",
+                    digits: 6,
+                    period: 30,
+                },
+            };
+        }
+    }
 }
 
 /**
@@ -1025,7 +1055,7 @@ async function clearUsageData() {
     // clear local storage
     localStorage.removeItem("foreignFills");
     localStorage.removeItem("recent");
-    Object.keys(localStorage).forEach(key => {
+    Object.keys(localStorage).forEach((key) => {
         if (key.startsWith("recent:")) {
             localStorage.removeItem(key);
         }
@@ -1069,38 +1099,6 @@ async function saveSettings(settings) {
     }
 }
 
-/**
- * Trigger OTP extension (browserpass-otp)
- *
- * @since 3.0.13
- *
- * @param object settings Settings object
- * @param string action   Browserpass action
- * @param object otp      OTP field data
- * @return void
- */
-function triggerOTPExtension(settings, action, otp) {
-    // trigger otp extension
-    for (let targetID of otpID) {
-        chrome.runtime
-            .sendMessage(targetID, {
-                version: chrome.runtime.getManifest().version,
-                action: action,
-                otp: otp,
-                settings: {
-                    host: settings.host,
-                    origin: settings.origin,
-                    tab: settings.tab
-                }
-            })
-            // Both response & error are noop functions, because we don't care about
-            // the response, and if there's an error it just means the otp extension
-            // is probably not installed. We can't detect that without requesting the
-            // management permission, so this is an acceptable workaround.
-            .then(noop => null, noop => null);
-    }
-}
-
 /**
  * Handle browser extension installation and updates
  *
@@ -1120,7 +1118,7 @@ function onExtensionInstalled(details) {
             title: title,
             message: message,
             iconUrl: "icon.png",
-            type: "basic"
+            type: "basic",
         });
     };
 
@@ -1139,10 +1137,10 @@ function onExtensionInstalled(details) {
             3002000: "New permissions added to clear copied credentials after 60 seconds.",
             3000000:
                 "New major update is out, please update the native host app to v3.\n" +
-                "Instructions here: https://github.com/browserpass/browserpass-native"
+                "Instructions here: https://github.com/browserpass/browserpass-native",
         };
 
-        var parseVersion = version => {
+        var parseVersion = (version) => {
             var [major, minor, patch] = version.split(".");
             return parseInt(major) * 1000000 + parseInt(minor) * 1000 + parseInt(patch);
         };
@@ -1151,7 +1149,7 @@ function onExtensionInstalled(details) {
 
         Object.keys(changelog)
             .sort()
-            .forEach(function(version) {
+            .forEach(function (version) {
                 if (prevVersion < version && newVersion >= version) {
                     show(version.toString(), "browserpass: Important changes", changelog[version]);
                 }
