@@ -1,12 +1,29 @@
-(function() {
+(function () {
     const FORM_MARKERS = ["login", "log-in", "log_in", "signin", "sign-in", "sign_in"];
     const OPENID_FIELDS = {
         selectors: ["input[name*=openid i]", "input[id*=openid i]", "input[class*=openid i]"],
-        types: ["text"]
+        types: ["text"],
     };
     const USERNAME_FIELDS = {
         selectors: [
             "input[autocomplete=username i]",
+
+            "input[name=login i]",
+            "input[name=user i]",
+            "input[name=username i]",
+            "input[name=email i]",
+            "input[name=alias i]",
+            "input[id=login i]",
+            "input[id=user i]",
+            "input[id=username i]",
+            "input[id=email i]",
+            "input[id=alias i]",
+            "input[class=login i]",
+            "input[class=user i]",
+            "input[class=username i]",
+            "input[class=email i]",
+            "input[class=alias i]",
+
             "input[name*=login i]",
             "input[name*=user i]",
             "input[name*=email i]",
@@ -19,26 +36,66 @@
             "input[class*=user i]",
             "input[class*=email i]",
             "input[class*=alias i]",
+
             "input[type=email i]",
+            "input[autocomplete=email i]",
             "input[type=text i]",
-            "input[type=tel i]"
+            "input[type=tel i]",
         ],
-        types: ["email", "text", "tel"]
+        types: ["email", "text", "tel"],
     };
     const PASSWORD_FIELDS = {
         selectors: [
             "input[type=password i][autocomplete=current-password i]",
-            "input[type=password i]"
-        ]
+            "input[type=password i]",
+        ],
     };
     const INPUT_FIELDS = {
         selectors: PASSWORD_FIELDS.selectors
             .concat(USERNAME_FIELDS.selectors)
-            .concat(OPENID_FIELDS.selectors)
+            .concat(OPENID_FIELDS.selectors),
     };
     const SUBMIT_FIELDS = {
         selectors: [
             "[type=submit i]",
+
+            "button[name=login i]",
+            "button[name=log-in i]",
+            "button[name=log_in i]",
+            "button[name=signin i]",
+            "button[name=sign-in i]",
+            "button[name=sign_in i]",
+            "button[id=login i]",
+            "button[id=log-in i]",
+            "button[id=log_in i]",
+            "button[id=signin i]",
+            "button[id=sign-in i]",
+            "button[id=sign_in i]",
+            "button[class=login i]",
+            "button[class=log-in i]",
+            "button[class=log_in i]",
+            "button[class=signin i]",
+            "button[class=sign-in i]",
+            "button[class=sign_in i]",
+            "input[type=button i][name=login i]",
+            "input[type=button i][name=log-in i]",
+            "input[type=button i][name=log_in i]",
+            "input[type=button i][name=signin i]",
+            "input[type=button i][name=sign-in i]",
+            "input[type=button i][name=sign_in i]",
+            "input[type=button i][id=login i]",
+            "input[type=button i][id=log-in i]",
+            "input[type=button i][id=log_in i]",
+            "input[type=button i][id=signin i]",
+            "input[type=button i][id=sign-in i]",
+            "input[type=button i][id=sign_in i]",
+            "input[type=button i][class=login i]",
+            "input[type=button i][class=log-in i]",
+            "input[type=button i][class=log_in i]",
+            "input[type=button i][class=signin i]",
+            "input[type=button i][class=sign-in i]",
+            "input[type=button i][class=sign_in i]",
+
             "button[name*=login i]",
             "button[name*=log-in i]",
             "button[name*=log_in i]",
@@ -74,8 +131,8 @@
             "input[type=button i][class*=log_in i]",
             "input[type=button i][class*=signin i]",
             "input[type=button i][class*=sign-in i]",
-            "input[type=button i][class*=sign_in i]"
-        ]
+            "input[type=button i][class*=sign_in i]",
+        ],
     };
 
     /**
@@ -89,7 +146,7 @@
     function fillLogin(request) {
         var result = {
             filledFields: [],
-            foreignFill: undefined
+            foreignFill: undefined,
         };
 
         // get the login form
@@ -163,9 +220,7 @@
      * @return object result of focusing or submitting a form
      */
     function focusOrSubmit(request) {
-        var result = {
-            needPressEnter: false
-        };
+        var result = {};
 
         // get the login form
         let loginForm = undefined;
@@ -201,11 +256,6 @@
                     submit.focus();
                 }
             } else {
-                // There is no submit button.
-                if (request.autoSubmit) {
-                    // signal background script that we want it to press Enter for us
-                    result.needPressEnter = true;
-                }
                 // We need to keep focus somewhere within the form, so that Enter hopefully submits the form.
                 for (let selectors of [OPENID_FIELDS, PASSWORD_FIELDS, USERNAME_FIELDS]) {
                     let field = find(selectors, loginForm);
@@ -263,7 +313,8 @@
                 if (
                     rect.x + rect.width < 0 ||
                     rect.y + rect.height < 0 ||
-                    (rect.x > window.innerWidth || rect.y > window.innerHeight)
+                    rect.x > window.innerWidth ||
+                    rect.y > window.innerHeight
                 ) {
                     continue;
                 }
@@ -382,6 +433,11 @@
      * @return bool Whether the update succeeded
      */
     function update(field, value, form) {
+        if (value === undefined) {
+            // undefined values should not be filled, but are always considered successful
+            return true;
+        }
+
         if (!value.length) {
             return false;
         }
@@ -411,6 +467,11 @@
             el.dispatchEvent(new Event(eventName, { bubbles: true }));
         }
 
+        // truncate the value if required by the field
+        if (el.maxLength > 0) {
+            value = value.substr(0, el.maxLength);
+        }
+
         // Set the field value
         let initialValue = el.value || el.getAttribute("value");
         el.setAttribute("value", value);
@@ -436,6 +497,6 @@
     // set window object
     window.browserpass = {
         fillLogin: fillLogin,
-        focusOrSubmit: focusOrSubmit
+        focusOrSubmit: focusOrSubmit,
     };
 })();
