@@ -61,7 +61,7 @@ function view(ctl, params) {
     nodes.push(
         m(
             "div.logins",
-            this.results.map(function(result) {
+            this.results.map(function (result) {
                 const storeBgColor = result.store.bgColor || result.store.settings.bgColor;
                 const storeColor = result.store.color || result.store.settings.color;
 
@@ -70,7 +70,7 @@ function view(ctl, params) {
                     {
                         key: result.index,
                         tabindex: 0,
-                        onclick: function(e) {
+                        onclick: function (e) {
                             var action = e.target.getAttribute("action");
                             if (action) {
                                 result.doAction(action);
@@ -78,16 +78,16 @@ function view(ctl, params) {
                                 result.doAction("fill");
                             }
                         },
-                        onkeydown: keyHandler.bind(result)
+                        onkeydown: keyHandler.bind(result),
                     },
                     [
-                        m("div.name", [
+                        m("div.name", { title: "Fill username / password | <Enter>" }, [
                             m("div.line1", [
                                 m(
                                     "div.store.badge",
                                     {
                                         style: `background-color: ${storeBgColor};
-                                                color: ${storeColor}`
+                                                color: ${storeColor}`,
                                     },
                                     result.store.name
                                 ),
@@ -100,22 +100,27 @@ function view(ctl, params) {
                                               " time" +
                                               (result.recent.count > 1 ? "s" : "") +
                                               ", last " +
-                                              Moment(new Date(result.recent.when)).fromNow()
+                                              Moment(new Date(result.recent.when)).fromNow(),
                                       })
-                                    : null
+                                    : null,
                             ]),
-                            m("div.line2", [m.trust(result.display)])
+                            m("div.line2", [m.trust(result.display)]),
                         ]),
-                        m("div.action.copy-password", {
-                            tabindex: 0,
-                            title: "Copy password",
-                            action: "copyPassword"
-                        }),
                         m("div.action.copy-user", {
                             tabindex: 0,
-                            title: "Copy username",
-                            action: "copyUsername"
-                        })
+                            title: "Copy username | <Ctrl+Shift+C>",
+                            action: "copyUsername",
+                        }),
+                        m("div.action.copy-password", {
+                            tabindex: 0,
+                            title: "Copy password | <Ctrl+C>",
+                            action: "copyPassword",
+                        }),
+                        m("div.action.details", {
+                            tabindex: 0,
+                            title: "Open Details | <Ctrl+O>",
+                            action: "getDetails",
+                        }),
                     ]
                 );
             })
@@ -130,10 +135,10 @@ function view(ctl, params) {
                     "a",
                     {
                         href: "https://github.com/browserpass/browserpass-native#installation",
-                        target: "_blank"
+                        target: "_blank",
                     },
                     "instructions"
-                )
+                ),
             ])
         );
     }
@@ -188,6 +193,8 @@ function keyHandler(e) {
                 e.target.querySelector(".action").focus();
             } else if (e.target.nextElementSibling) {
                 e.target.nextElementSibling.focus();
+            } else {
+                this.doAction("getDetails");
             }
             break;
         case "ArrowLeft":
@@ -218,5 +225,25 @@ function keyHandler(e) {
                 this.doAction(e.shiftKey ? "launchInNewTab" : "launch");
             }
             break;
+        case "KeyO":
+            if (e.ctrlKey) {
+                this.doAction("getDetails");
+            }
+            break;
+        case "Home": {
+            document.querySelector(".part.search input[type=text]").focus();
+            document.querySelector(".logins").scrollTo(0, 0);
+            window.scrollTo(0, 0);
+            break;
+        }
+        case "End": {
+            let logins = document.querySelectorAll(".login");
+            if (logins.length) {
+                let target = logins.item(logins.length - 1);
+                target.focus();
+                target.scrollIntoView();
+            }
+            break;
+        }
     }
 }
