@@ -1,5 +1,6 @@
 module.exports = SearchInterface;
 
+const BrowserpassURL = require("@browserpass/url");
 const m = require("mithril");
 
 /**
@@ -29,11 +30,11 @@ function SearchInterface(popup) {
  */
 function view(ctl, params) {
     var self = this;
-    var host = new URL(this.popup.settings.origin).host;
+    var host = new BrowserpassURL(this.popup.settings.origin).host;
     return m(
         "form.part.search",
         {
-            onkeydown: function(e) {
+            onkeydown: function (e) {
                 switch (e.code) {
                     case "Tab":
                         e.preventDefault();
@@ -55,34 +56,35 @@ function view(ctl, params) {
                         }
                         break;
                 }
-            }
+            },
         },
         [
             this.popup.currentDomainOnly
                 ? m("div.hint.badge", [
                       host,
                       m("div.remove-hint", {
-                          onclick: function(e) {
+                          title: "Clear domain filter | <Backspace>",
+                          onclick: function (e) {
                               var target = document.querySelector(
                                   ".part.search > input[type=text]"
                               );
                               target.focus();
                               self.popup.currentDomainOnly = false;
                               self.popup.search(target.value);
-                          }
-                      })
+                          },
+                      }),
                   ])
                 : null,
             m("input[type=text]", {
                 focused: true,
                 placeholder: "Search logins...",
-                oncreate: function(e) {
+                oncreate: function (e) {
                     e.dom.focus();
                 },
-                oninput: function(e) {
+                oninput: function (e) {
                     self.popup.search(e.target.value);
                 },
-                onkeydown: function(e) {
+                onkeydown: function (e) {
                     switch (e.code) {
                         case "Backspace":
                             if (self.popup.currentDomainOnly) {
@@ -114,9 +116,26 @@ function view(ctl, params) {
                                 );
                             }
                             break;
+                        case "KeyO":
+                            if (e.ctrlKey && e.target.selectionStart == e.target.selectionEnd) {
+                                e.preventDefault();
+                                self.popup.results[0].doAction("getDetails");
+                            }
+                            break;
+                        case "End": {
+                            if (e.target.selectionStart === e.target.value.length) {
+                                let logins = document.querySelectorAll(".login");
+                                if (logins.length) {
+                                    let target = logins.item(logins.length - 1);
+                                    target.focus();
+                                    target.scrollIntoView();
+                                }
+                            }
+                            break;
+                        }
                     }
-                }
-            })
+                },
+            }),
         ]
     );
 }
